/*
Programa kum lekciite po OOP na doc. Buchvarov
Napisana ot men, Ivan Dimitrov Georgiev (veche zavurshil) student po informatika, e-mail ivandg@yahoo.com
*/
#include <iostream.h>
#include <string.h>
#include <assert.h>
#include <iomanip.h>
class Employee {
 public:
   Employee (const char *, const char *);
   ~Employee ();
   const char *getFirstName () const;
   const char *getLastName () const;
   virtual float earnings () const = 0;
   virtual void print () const = 0;
 private:
   char *firstName;
   char *lastName;
} ;
Employee::Employee (const char *first, const char *last)
 { firstName = new char [strlen (first) + 1];
   assert (firstName != NULL);
   strcpy (firstName, first);
   lastName = new char [strlen (last) + 1];
   assert (lastName != NULL);
   strcpy (lastName, last);
 }
Employee::~Employee ()
 { delete []firstName;
   delete []lastName;
 }
const char *Employee::getFirstName () const
 { return firstName;}
const char *Employee::getLastName () const
 { return lastName;}
class Boss: public Employee {
 public:
   Boss ( const char *, const char *, float = 0);
   void setWeeklySalary (float);
   virtual float earnings () const;
   virtual void print () const;
 private:
   float weeklySalary;
} ;

Boss::Boss (const char *first, const char *last, float s) : 
Employee (first, last)
 { setWeeklySalary (s); }
void Boss::setWeeklySalary (float s)
{ weeklySalary = (s > 0) ? s : 0; }
float Boss::earnings () const
{ return weeklySalary; }
void Boss::print () const
{ cout << " : " << getFirstName () << ' '
         << getLastName ();
}
class CommissionWorker: public Employee {
 public:
   CommissionWorker ( const char *, const char *, float = 0, float = 0,
   unsigned = 0);
   void setSalary (float);
   void setCommission (float);
   void setQuantity (unsigned);
   virtual float earnings () const;
   virtual void print () const;
 private:
   float salary;
   float commission;
   unsigned quantity;
} ;
CommissionWorker:: CommissionWorker (const char *first, const char *last, float s, float c, unsigned q) : Employee (first, last)
 { setSalary (s); setCommission (c); setQuantity (q); }
void CommissionWorker::setSalary (float s)
{ salary = (s > 0) ? s : 0; }
void CommissionWorker::setCommission (float c)
{ commission = (c > 0) ? c : 0; }
void CommissionWorker::setQuantity (unsigned q)
{ quantity = (q > 0) ? q : 0; }
float CommissionWorker::earnings () const
{ return salary + commission*quantity; }
void CommissionWorker::print () const
{ cout << " : " << getFirstName () << ' '
         << getLastName ();
}
class PieceWorker: public Employee {
 public:
   PieceWorker ( const char *, const char *, float = 0, unsigned = 0);
   void setWagePerPiece (float);
   void setQuantity (unsigned);
   virtual float earnings () const;
   virtual void print () const;
 private:
   float wagePerPiece;
   unsigned quantity;
} ;
PieceWorker:: PieceWorker (const char *first, const char *last, float w,
unsigned q) : Employee (first, last)
 { setWagePerPiece (w); setQuantity (q); }
void PieceWorker::setWagePerPiece (float w)
{ wagePerPiece = (w > 0) ? w : 0; }
void PieceWorker::setQuantity (unsigned q)
{ quantity = (q > 0) ? q : 0; }
float PieceWorker::earnings () const
{ return wagePerPiece*quantity; }
void PieceWorker::print () const
{ cout << " : " << getFirstName () << ' '
         << getLastName ();
}
class HourlyWorker: public Employee {
 public:
   HourlyWorker ( const char *, const char *, float = 0, float = 0);
   void setWage (float);
   void setHours (float);
   virtual float earnings () const;
   virtual void print () const;
 private:
   float wage;
   float hours;
} ;
HourlyWorker:: HourlyWorker (const char *first, const char *last, float w,
float h) : Employee (first, last)
 { setWage (w); setHours (h); }
void HourlyWorker::setWage (float w)
{ wage = (w > 0) ? w : 0; }
void HourlyWorker::setHours (float h)
{ hours = (h > 0 && h <=168 ) ? h : 0; }
float HourlyWorker::earnings () const
{ return (hours < 40) ? wage*hours: 40*wage + 1.5*(hours-40)*wage; }
void HourlyWorker::print () const
{ cout << " : " << getFirstName () << ' '
         << getLastName ();
}
void main ()
{
 cout << setiosflags (ios::fixed | ios::showpoint) << setprecision (2);
 Employee *ptr;
 Boss b ("", "", 800);
 ptr = &b;
 ptr -> print ();// 
 cout << " $" << ptr -> earnings () << endl;// 
 b.print ();// 
 cout << " $" << b.earnings () << endl;// 
 CommissionWorker c ("", "", 200, 3, 150);
 ptr = &c;
 ptr -> print ();
 cout << " $" << ptr -> earnings () << endl;
 c.print ();
 cout << " $" << c.earnings () << endl;
 PieceWorker p ("", "", 200, 12);
 ptr = &p;
 ptr -> print ();
 cout << " $" << ptr -> earnings () << endl;
 p.print ();
 cout << " $" << p.earnings () << endl;
 HourlyWorker h ("", "", 100, 55);
 ptr = &h;
 ptr -> print ();
 cout << " $" << ptr -> earnings () << endl;
 h.print ();
 cout << " $" << h.earnings () << endl;
 char cc;
 cin >> cc;
}
